unit voxel1;

interface
{****************************************************************************
** Demonstration of displaying a voxel heightscape                         **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.home.ml.org                                               **
**                                                                         **
****************************************************************************}

{ Delphi version by Paul TOTH <tothpaul@multimania.com> }

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DIBS, ExtCtrls;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Timer1Timer(Sender: TObject);
    procedure FormClick(Sender: TObject);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
    DIB:TDIB256;
    x,y,angle:word;
    cx,cy:integer;
    procedure initmap;
    procedure drawmap;
    procedure VLine(x,y1,y2,c:integer);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
const
 {This controls the maximum distance displayed. If you increase this,
  you will see more of the landscape, but it will also be slower}
 Depth = 500;
 maxx=320;
 maxy=200;

Var
 {Cosine and Sine tables}
 CosT, SinT : Array [0..2047] of Integer;
 {Distance compensation table}
 DComp : Array [1..Depth + 1] of Integer;

 iMap:array[0..$FFFF] of byte;
 hMap:array[0..$FFFF] of byte;

procedure TForm1.InitMap;
var
  DAC: Byte;
  f : File;
  colors:array[0..255,0..3] of byte;
begin
  {Load in the file "GROUND.BMP"}
  AssignFile(f, 'GROUND.BMP');
  Reset(f, 1);
  Seek (f, 54);
  blockread(f,colors,sizeof(colors));
  for DAC:=0 to 255 do DIB.Color[DAC]:=RGB(Colors[DAC,2],Colors[DAC,1],Colors[DAC,0]);
  DIB.UpdateColors;
  blockread(f,imap,SizeOf(imap));
  CloseFile(f);

  {Load in the file "HEIGHT.BMP"}
  AssignFile(f, 'HEIGHT.BMP');
  Reset(f, 1);
  Seek (f, 1078);
  blockread(f,hmap,sizeof(hmap));
  CloseFile(f);
end;

procedure TForm1.FormCreate(Sender: TObject);
 const
  k=256;
 var
  a:integer;
 begin
  DIB:=TDIB256.Create(Self);
  DIB.Width:=maxx;
  DIB.Height:=maxy;
  DIB.CreateDIB;
 { calculates some lookup table}
  for a := 0 to 2047 do begin
  {Precalculate cosine}
   CosT [a] := trunc(Cos (a * pi / 1024) * k);
  {and sine}
   SinT [a] := trunc(Sin (a * pi / 1024) * k);
  end;
 {Precalculate distance compensation table}
  for a := 1 to Depth + 1 do DComp [a] := 1000 div longint(a) + maxy div 2;

 {Set up player}
  x := $8000;
  y := $8000;
  Angle := 600;

  {Create the map}
  InitMap;
 end;

procedure TForm1.DrawMap;
 var
  height:integer;
  i:integer;
  a:word;
  px,py,c:integer;
  deltax,deltay:integer;
  miny,maxy:integer;
  d:integer;
  h:integer;
  y1:integer;
 begin
  height:=hmap[y and $FF00+hi(x)];
  for i:=0 to maxx-1 do begin
   a:=(angle+i+1888) and 2047;
   px:=x;
   py:=y;
   deltax := CosT [a];
   deltay := SinT [a];
   minY := maxy;
   for d:=1 to Depth do begin
    inc (px, deltax);
    inc (py, deltay);
    h := hmap[py and $FF00+hi(px)]-height;
    c := imap[py and $FF00+hi(px)];
    y1 := DComp [d] - (h shl 5) div d;
    if y1 < minY then begin
     VLine (i, y1, minY,c);
     minY := y1;
     if miny=0 then break;
    end;
   end;
   if MinY>0 then VLine(i,0,minY,128+32+16);
  end;

 end;

procedure TForm1.VLine(x,y1,y2,c:integer);
 var
  y:integer;
 begin
  if y1<0 then y1:=0;
  if y2>=maxy then y2:=maxy-1;
  for y:=y1 to y2 do DIB.Putpixel(x,y,c);
 end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 drawmap;
 DIB.DrawXY(Canvas.Handle,cx,cy);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
 cx:=(ClientWidth-maxx)div 2;
 cy:=(ClientHeight-maxy) div 2;
 invalidate;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
const
 k=1;
begin
 case key of
  vk_up  : begin {move forward}
             inc (x, k*CosT [Angle]);
             inc (y, k*SinT [Angle]);
           end;
  vk_left: begin {turn left}
             Angle := (Angle + 2032) and 2047;
           end;
  vk_right: begin {turn right}
             Angle := (Angle + 16) and 2047;
           end;
  vk_down: begin {move backward}
             dec (x, k*CosT [Angle]);
             dec (y, k*SinT [Angle]);
           end;
  else exit;
 end;
 Paint;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
const
 rotate:integer=0;
var
 key:word;
begin
 if rotate=0 then
 case random(100) of
  50..74: inc(rotate,random(10));
  75..99: dec(rotate,random(10));
 end;
 if rotate>0 then begin
  Angle := (Angle + 2032) and 2047;
  dec(rotate)
 end;
 if rotate<0 then begin
  Angle := (Angle + 16) and 2047;
  inc(rotate)
 end;
 inc (x, CosT [Angle]);
 inc (y, SinT [Angle]);
 Paint;
end;

procedure TForm1.FormClick(Sender: TObject);
begin
 Timer1.Enabled:=not Timer1.Enabled;
end;

end.
